Module: Datadog::Profiling::NativeExtensionHelpers

Defined in:
ext/datadog_profiling_native_extension/native_extension_helpers.rb

Overview

Helpers for extconf.rb

Defined Under Namespace

Modules: Supported

Constant Summary collapse

ENV_NO_EXTENSION =

Can be set when customers want to skip compiling the native extension entirely

"DD_PROFILING_NO_EXTENSION"
ENV_FAIL_INSTALL_IF_MISSING_EXTENSION =

Can be set to force rubygems to fail gem installation when profiling extension could not be built

"DD_PROFILING_FAIL_INSTALL_IF_MISSING_EXTENSION"

Class Method Summary collapse

Class Method Details

.fail_install_if_missing_extension? ⇒ Boolean

Returns:

  • (Boolean)


12
13
14
# File 'ext/datadog_profiling_native_extension/native_extension_helpers.rb', line 12

def self.fail_install_if_missing_extension?
  ENV[ENV_FAIL_INSTALL_IF_MISSING_EXTENSION].to_s.strip.downcase == "true"
end