Class: Datadog::Tracing::Configuration::Dynamic::Option
- Inherits:
-
Object
- Object
- Datadog::Tracing::Configuration::Dynamic::Option
- Defined in:
- lib/datadog/tracing/configuration/dynamic/option.rb
Overview
Maps a remote dynamic configuration to a location configuration option.
Direct Known Subclasses
Instance Attribute Summary collapse
-
#env_var ⇒ Object
readonly
Returns the value of attribute env_var.
-
#name ⇒ Object
readonly
Returns the value of attribute name.
Instance Method Summary collapse
-
#call(value) ⇒ Object
Reconfigures the provided option, setting its value to
value. -
#initialize(name, env_var) ⇒ Option
constructor
A new instance of Option.
Constructor Details
#initialize(name, env_var) ⇒ Option
Returns a new instance of Option.
14 15 16 17 |
# File 'lib/datadog/tracing/configuration/dynamic/option.rb', line 14 def initialize(name, env_var) @name = name @env_var = env_var end |
Instance Attribute Details
#env_var ⇒ Object (readonly)
Returns the value of attribute env_var.
11 12 13 |
# File 'lib/datadog/tracing/configuration/dynamic/option.rb', line 11 def env_var @env_var end |
#name ⇒ Object (readonly)
Returns the value of attribute name.
11 12 13 |
# File 'lib/datadog/tracing/configuration/dynamic/option.rb', line 11 def name @name end |
Instance Method Details
#call(value) ⇒ Object
Reconfigures the provided option, setting its value to value.
22 23 24 |
# File 'lib/datadog/tracing/configuration/dynamic/option.rb', line 22 def call(value) raise NotImplementedError end |