Module: Datadog::Tracing::Sampling::Ext::Decision

Defined in:
lib/datadog/tracing/sampling/ext.rb

Overview

List of how the decision was made for the trace-level sampling.

These values used to populate the Metadata::Ext::Distributed::TAG_DECISION_MAKER tag.

The decision has two parts, separated by a -: part1-sampling_mechanism. part1 is currently not populated, thus this tag is currently formatted as "-sampling_mechanism".

Constant Summary collapse

DEFAULT =

Used before the tracer receives any rates from agent and there are no rules configured.

"-0"
AGENT_RATE =

The sampling rate received in the agent's http response.

"-1"
TRACE_SAMPLING_RULE =

Locally configured rule.

"-3"
MANUAL =

User directly sets sampling priority via Datadog::Tracing.reject! or Datadog::Tracing.keep!, or by a custom sampler implementation.

"-4"
ASM =

Formerly AppSec.

"-5"
AI_GUARD =

AI Guard.

"-13"
REMOTE_USER_RULE =

Dynamically configured rule, explicitly created by the user.

"-11"
REMOTE_DYNAMIC_RULE =

Dynamically configured rule, automatically generated by Datadog.

"-12"