Class: Datadog::Tracing::TraceOperation::Events::TraceResourceChange

Inherits:
Event
  • Object
show all
Defined in:
lib/datadog/tracing/trace_operation.rb

Overview

Triggered when the resource name is set This is used to reconsider trace sampling on resource name change

Instance Attribute Summary

Attributes inherited from Event

#name, #subscriptions

Instance Method Summary collapse

Methods inherited from Event

#publish, #subscribe, #unsubscribe_all!

Constructor Details

#initializeTraceResourceChange

Returns a new instance of TraceResourceChange.



537
538
539
# File 'lib/datadog/tracing/trace_operation.rb', line 537

def initialize
  super(:trace_resource_change)
end