Class: Datadog::DI::Error::ProbeTargetForbidden Private
- Inherits:
-
Error
- Object
- Error
- Datadog::DI::Error::ProbeTargetForbidden
- Defined in:
- lib/datadog/di/error.rb
Overview
This class is part of a private API. You should avoid using this class if possible, as it may be removed or be changed in the future.
Raised when a probe targets a class or module that the tracer refuses to instrument (for example, anything in the Datadog namespace, which the tracer itself uses internally).