Module: Datadog::Tracing::Contrib::Propagation::SqlComment::Ext
- Defined in:
- lib/datadog/tracing/contrib/propagation/sql_comment/ext.rb
Constant Summary collapse
- ENV_DBM_PROPAGATION_MODE =
'DD_DBM_PROPAGATION_MODE'- ENV_DBM_INJECT_SQL_BASEHASH =
Determines whether to inject the propagation hash into the SQL comment TODO: BASEHASH is confusing as a name, so look into better names that can apply cross tracer in the future
'DD_DBM_INJECT_SQL_BASEHASH'- DISABLED =
The default mode for sql comment propagation
'disabled'- SERVICE =
The
servicemode propagates service configuration 'service'- FULL =
The
fullmode propagates service configuration + trace context 'full'- TAG_DBM_TRACE_INJECTED =
The value should be
truewhenfullmode '_dd.dbm_trace_injected'- TAG_PROPAGATED_HASH =
Checksum of the agent’s container tags and this process’ tags
'_dd.propagated_hash'- KEY_DATABASE_SERVICE =
Database service/sql span service (i.e. the service executing the actual query)
If fake services are disabled:
This value will be the same as the parent serviceIf fake services are enabled:
This value is NOT the same as the parent serviceThis should NOT be overridden by peer.service.
'dddbs'- KEY_ENVIRONMENT =
The global service environment (e.g. DD_ENV)
'dde'- KEY_PARENT_SERVICE =
The global service name (e.g. DD_SERVICE)
'ddps'- KEY_VERSION =
The global service version (e.g. DD_VERSION)
'ddpv'- KEY_HOSTNAME =
The hostname of the database server, as provided to the database client upon instantiation.
'ddh'- KEY_DB_NAME =
'dddb'- KEY_PEER_SERVICE =
Users can use this attribute to specify the identity of the dependency/database they are connecting to. We should grab this attribute only if the user is EXPLICITLY specifying it.
'ddprs'- KEY_BASE_HASH =
DBM service hash (ddsh) for propagation
'ddsh'- KEY_TRACEPARENT =
'traceparent'