Class: Datadog::Tracing::TraceOperation::Events::TraceResourceChange
- Defined in:
- lib/datadog/tracing/trace_operation.rb
Overview
Triggered when the resource name is set This is used to reconsider trace sampling on resource name change
Instance Attribute Summary
Attributes inherited from Event
Instance Method Summary collapse
-
#initialize ⇒ TraceResourceChange
constructor
A new instance of TraceResourceChange.
Methods inherited from Event
#publish, #subscribe, #unsubscribe_all!
Constructor Details
#initialize ⇒ TraceResourceChange
Returns a new instance of TraceResourceChange.
537 538 539 |
# File 'lib/datadog/tracing/trace_operation.rb', line 537 def initialize super(:trace_resource_change) end |